  function [x,resvec] = mygmres(A,b,x0,tol,maxit)
% GMRES method for the solution of a linear system
% Inputs:    A - system matrix
%            b - right hand side
%           x0 - initial guess
%          tol - method stops if the maximum norm of the residual
%                b-A*x is less than tol
%        maxit - maximum number of iterations
%
% Outputs:   x - solution vector
%       resvec - residual vector

  n = size(A,1); 

% Init matrices and vectors  
  V = zeros(n,maxit+1);
  H = zeros(maxit+1,maxit);

  gamma = zeros(maxit+1,1);
  c = zeros(maxit+1,1);
  s = zeros(maxit+1,1);

  resvec = zeros(maxit+1,1);

% Init GMRES
  r0 = b-A*x0;
  normr0 = norm(r0);
  V(:,1) = r0/normr0;
  gamma(1) = normr0;

  normres = gamma(1);
  resvec(1) = normres;
  
  j = 1;
  while normres > tol && j <= maxit
    
    % Arnoldi method
    w = A*V(:,j);
    for i=1:j
      H(i,j) = V(:,i)'*w;
      w = w - V(:,i)*H(i,j);
    end
    normw = norm(w);
    H(j+1,j) = normw;
    
    
    % Givens Rotation
    for i=1:j-1
      %H(i:i+1,j) = [c(i) s(i); -s(i) c(i)]*H(i:i+1,j);
      h1 = H(i,j);
      h2 = H(i+1,j);
      H(i,j)   =  c(i)*h1 + s(i)*h2;
      H(i+1,j) = -s(i)*h1 + c(i)*h2;
    end
    beta = norm(H(j:j+1,j));
    s(j) = H(j+1,j)/beta;
    c(j) = H(j,j)/beta;
    
    % Store matrix R in H
    H(j,j) = beta;
    H(j+1,j) = 0;
    
    % Update "residual" vector
    gamma(j+1) = -s(j)*gamma(j);
    gamma(j) = c(j)*gamma(j);
    
    V(:,j+1) = w/normw;
    
    normres = abs(gamma(j+1));
    
    j = j+1;
    resvec(j) =  normres;
  end

% Compuation of approximate solution x
  idx = 1:j-1;
  alpha = H(idx,idx)\gamma(idx); % backward substitution
  x = x0 + V(:,idx)*alpha;

% shorten resvec
  resvec = resvec(1:j); 

  
  